﻿using System.Windows;
using PointSnapToLayer;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace PreThinkGeo
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
			InitializeComponent();
		}

		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
			wpfMap1.CurrentExtent = new RectangleShape(-97.7591, 30.3126, -97.7317, 30.2964);
			//this.wpfMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 198, 255, 255));

			//Генерируем некоторый полигон, добавляем его в слой, слой добавляем в обертку, обертку добавляем к карте

			#region Здесь создаем слой с полигоном (допустим, это некое здание)

			var polygonInMemoryFeatureLayer = new InMemoryFeatureLayer();
			//polygonInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Park1;
			polygonInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color =
				GeoColor.SimpleColors.Red;
			polygonInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
			polygonInMemoryFeatureLayer.InternalFeatures.Add("Polygon",
			                                                 new Feature
			                                                 	(
			                                                 	BaseShape.CreateShapeFromWellKnownData
			                                                 		(
			                                                 			@"POLYGON((-97.7385464920044 30.3131045265198,
                                -97.7342549575806 30.3107441825867,
                                -97.7392331375122 30.3030194206238, 
                                -97.7437821640015 30.3052510185242, 
                                -97.7385464920044 30.3131045265198))"
			                                                 		)
			                                                 	)
				);

			var inMemoryOverlay = new LayerOverlay();
			inMemoryOverlay.Layers.Add("InMemoryFeatureLayer", polygonInMemoryFeatureLayer);
			wpfMap1.Overlays.Add("InMemoryOverlay", inMemoryOverlay);

			#endregion

			//создаем наш собственный слой
			var snapToLayerEditInteractiveOverlay = new SnapToLayerEditInteractiveOverlay();
			//устанавливаем размер области, в пределах которой будет происходить притяжка
			snapToLayerEditInteractiveOverlay.Tolerance = 25;
			//устанавливаем единицы измерения
			snapToLayerEditInteractiveOverlay.ToleranceUnit = DistanceUnit.Meter;

			snapToLayerEditInteractiveOverlay.EditShapesLayer.InternalFeatures.Add("MultiLine",
			                                                                       new Feature
			                                                                       	(
			                                                                       	BaseShape.CreateShapeFromWellKnownData
			                                                                       		(
			                                                                       			@"LINESTRING(
                                    -97.74663 30.30911,
                                    -97.74653 30.30774,
                                    -97.74635 30.30658, 
                                    -97.74819 30.30589,
                                    -97.74688 30.30477,
                                    -97.74656 30.30366,
                                    -97.74662 30.30319)"
			                                                                       		)
			                                                                       	)
				);


			//задаем стили под перетаскиваемые точки и под точки объекта, к которому осуществляем привязку (для 
			snapToLayerEditInteractiveOverlay.ControlPointStyle = new PointStyle(PointSymbolType.Circle,
			                                                                     new GeoSolidBrush(
			                                                                     	GeoColor.StandardColors.PaleGoldenrod),
			                                                                     new GeoPen(GeoColor.StandardColors.Black), 8);
			snapToLayerEditInteractiveOverlay.DraggedControlPointStyle = new PointStyle(PointSymbolType.Circle,
			                                                                            new GeoSolidBrush(
			                                                                            	GeoColor.StandardColors.Red),
			                                                                            new GeoPen(
			                                                                            	GeoColor.StandardColors.Orange, 2), 10);

			//устанавливаем слой, к которому будем осуществлять притяжку
			snapToLayerEditInteractiveOverlay.ToSnapInMemoryFeatureLayer = polygonInMemoryFeatureLayer;
			//подсчитываем все контрольные точки
			snapToLayerEditInteractiveOverlay.CalculateAllControlPoints();

			//задаем у карты в качестве слоя для редактирования наш созданный слой
			wpfMap1.EditOverlay = snapToLayerEditInteractiveOverlay;
			wpfMap1.Refresh();
		}

		private bool _isTsquareOn;
		private void checkBox1_Checked(object sender, RoutedEventArgs e)
		{
			_isTsquareOn = checkBox1.IsChecked.Value;
		}

		private void wpfMap1_MapClick(object sender, MapClickWpfMapEventArgs e)
		{
			string s;
			s = "s";
		}
	}
}