﻿
using System;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Windows.Input;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace PointSnapToLayer
{
	internal class SnapToLayerEditInteractiveOverlay : EditInteractiveOverlay
	{
		#region ctor

		public SnapToLayerEditInteractiveOverlay()
		{

		}

		#endregion

		private PointStyle _controlPointStyle;
		private PointStyle _draggedControlPointStyle;
		private GeographyUnit _geographyUnit;
		private InMemoryFeatureLayer _toSnapInMemoryFeatureLayer;
		private float _tolerance;

		private DistanceUnit _toleranceUnit;

		public PointStyle ControlPointStyle
		{
			get { return _controlPointStyle; }
			set { _controlPointStyle = value; }
		}

		public PointStyle DraggedControlPointStyle
		{
			get { return _draggedControlPointStyle; }
			set { _draggedControlPointStyle = value; }
		}

		public InMemoryFeatureLayer ToSnapInMemoryFeatureLayer
		{
			get { return _toSnapInMemoryFeatureLayer; }
			set { _toSnapInMemoryFeatureLayer = value; }
		}

		public float Tolerance
		{
			get { return _tolerance; }
			set { _tolerance = value; }
		}

		public DistanceUnit ToleranceUnit
		{
			get { return _toleranceUnit; }
			set { _toleranceUnit = value; }
		}

		/// <summary>
		/// Переопределяем движение вершины 
		/// </summary>
		/// <param name="sourceFeature">Исходный объект</param>
		/// <param name="sourceControlPoint">Исходная точка</param>
		/// <param name="targetControlPoint">Конечная точка</param>
		/// <returns></returns>
		protected override Feature MoveVertexCore(Feature sourceFeature, PointShape sourceControlPoint,
												  PointShape targetControlPoint)
		{
			var vertexMovingEditInteractiveOverlayEventArgs = new VertexMovingEditInteractiveOverlayEventArgs(false,
																											  sourceFeature,
																											  new Vertex(
																												targetControlPoint));
			PointShape snapPointShape = null;
			//определяем близлежащую точку
			if (Keyboard.IsKeyDown(Key.LeftCtrl))
				snapPointShape = FindNearestSnappingPoint(targetControlPoint);

			if (snapPointShape != null)
				vertexMovingEditInteractiveOverlayEventArgs.MovingVertex = new Vertex(snapPointShape);

			return base.MoveVertexCore(sourceFeature, sourceControlPoint,
									   new PointShape(vertexMovingEditInteractiveOverlayEventArgs.MovingVertex));

		}

		/// <summary>
		/// Функция для нахождения близлежащей точки
		/// </summary>
		/// <param name="targetPointShape"></param>
		/// <returns></returns>
		private PointShape FindNearestSnappingPoint(PointShape targetPointShape)
		{
			_toSnapInMemoryFeatureLayer.Open();
			Collection<Feature> toSnapInMemoryFeatures
				= _toSnapInMemoryFeatureLayer.FeatureSource.GetFeaturesNearestTo(targetPointShape, GeographyUnit.Meter, 1,
																				ReturningColumnsType.AllColumns);

			_toSnapInMemoryFeatureLayer.Close();

			if (toSnapInMemoryFeatures.Count == 1)
			{
				var polygonShape = (PolygonShape)toSnapInMemoryFeatures[0].GetShape();

				foreach (Vertex vertex in polygonShape.OuterRing.Vertices)
				{
					var toSnapPointShape = new PointShape(vertex);
					double Distance = toSnapPointShape.GetDistanceTo(targetPointShape, _geographyUnit, _toleranceUnit);

					if (Distance <= _tolerance)
					{
						return new PointShape(toSnapPointShape.X, toSnapPointShape.Y);
					}
				}
			}
			return null;
		}

		/// <summary>
		/// Переопределяем функцию отрисовки
		/// </summary>
		/// <param name="canvas"></param>
		protected override void DrawTileCore(GeoCanvas canvas)
		{
			//Устанавливаем необходимые параметры
			_geographyUnit = canvas.MapUnit;

			var labelsInAllLayers = new Collection<SimpleCandidate>();
			EditShapesLayer.Open();
			EditShapesLayer.Draw(canvas, labelsInAllLayers);
			canvas.Flush();

			ExistingControlPointsLayer.Open();
			Collection<Feature> controlPoints =
				ExistingControlPointsLayer.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);

			foreach (Feature feature in controlPoints)
			{
				if (feature.ColumnValues["state"] != "selected")
				{
					var features = new Feature[1] { feature };
					_controlPointStyle.Draw(features, canvas, labelsInAllLayers, labelsInAllLayers);
				}
				else
				{
					var features = new Feature[1] { feature };
					_draggedControlPointStyle.Draw(features, canvas, labelsInAllLayers, labelsInAllLayers);
				}
			}
		}

	}
}